<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 18.09.18
 * Time: 11:32
 */

namespace WS\EMarketing\Request;


use WS\EMarketing\CaseConverter;

/**
 * Class AbstractRequest
 * @package WS\EMarketing\Request
 */
abstract class AbstractRequest implements \JsonSerializable
{

    use CaseConverter;

    const METHOD_POST = 'POST';
    const METHOD_GET = 'GET';
    const METHOD_PUT = 'PUT';

    const ENDPOINT = null;
    const METHOD = self::METHOD_GET;

    /**
     * @return string|null
     */
    public function getEndpoint()
    {
        return static::ENDPOINT;
    }

    /**
     * @return bool
     */
    public function getMethod()
    {
        return static::METHOD;
    }

    /**
     * Checks if request is valid
     *
     * @return bool
     */
    abstract public function isValid(&$errorMessage);

    /**
     * @return bool
     */
    public function isPost()
    {
        return static::METHOD === self::METHOD_POST || static::METHOD === self::METHOD_PUT;
    }

    /**
     * @return string
     */
    public function buildQuery()
    {
        $data = [];
        foreach (get_object_vars($this) as $prop => $value) {
            if (is_string($value)) {
                $value = utf8_encode($value);
            }
            $data[self::fromCamelCase($prop)] = $value;
        }
        return http_build_query($data);
    }

    /**
     * Specify data which should be serialized to JSON
     * @link https://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        $data = [];
        foreach (get_object_vars($this) as $prop => $value) {
            if ($this->$prop !== null && $prop[0] !== '_') {
                if ($value instanceof \DateTime) {
                    $data[self::fromCamelCase($prop)] = $value->format('Y-m-d');
                    continue;
                }
                if (is_array($value) && !count($value)) {
                    continue;
                }

                if (is_string($value)) {
                    $value = utf8_encode($value);
                }

                $data[self::fromCamelCase($prop)] = $value;
            }
        }
        return $data;
    }

    /**
     * @return \WS\EMarketing\Response\AbstractResponse
     */
    abstract public function getResponseObject();

}