<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 18.09.18
 * Time: 11:32
 */

namespace WS\EMarketing\Request;


class Signup extends AbstractRequest
{
    /**
     * TODO: SANDBOX MODE
     */
    const ENDPOINT = 'https://app.emarketing.com/partner_api/signup';
    const METHOD = self::METHOD_POST;

    const AUTH_SHOPSYSTEM_API = 'shopsystem_api';

    /**
     * Valid values for $country
     *
     * @var array
     */
    public static $validCountries = ['US', 'GB', 'DE', 'FR', 'JP', 'IT', 'ES', 'NL', 'BR',
        'AU', 'CH', 'AT', 'BE', 'DK', 'MX', 'NO', 'PL', 'SE', 'TR', 'CZ', 'CA', 'IN', 'RU'];

    /**
     * Valid values for $currencyCode
     *
     * @var array
     */
    public static $validCurrencyCodes = ['AED', 'ARS', 'AUD', 'BGN', 'BND', 'BOB', 'BRL',
        'CAD', 'CHF', 'CLP', 'CNY', 'COP', 'CSD', 'CZK', 'DEM', 'DKK', 'EEK', 'EGP', 'EUR',
        'FJD', 'FRF', 'GBP', 'HKD', 'HRK', 'HUF', 'IDR', 'ILS', 'INR', 'JOD', 'JPY', 'KES',
        'KRW', 'LKR', 'LTL', 'MAD', 'MTL', 'MXN', 'MYR', 'NGN', 'NOK', 'NZD', 'PEN', 'PHP',
        'PKR', 'PLN', 'ROL', 'RON', 'RSD', 'RUB', 'SAR', 'SEK', 'SGD', 'SIT', 'SKK', 'THB',
        'TND', 'TRL', 'TRY', 'TWD', 'UAH', 'USD', 'UYU', 'VEB', 'VEF', 'VND', 'ZAR'];

    /**
     * Initially selected AdWords account ID
     *
     * @example 1234561234
     * @var int|null
     */
    public $adwordsId;

    /**
     * Specify additional APIs you want to use here
     *
     * @var array
     */
    public $authorizations = [self::AUTH_SHOPSYSTEM_API];

    /**
     * Initially selected country one of:'US' or 'GB' or 'DE' or 'FR' or 'JP' or 'IT' or
     * 'ES' or 'NL' or 'BR' or 'AU' or 'CH' or 'AT' or 'BE' or 'DK' or 'MX' or 'NO' or
     * 'PL' or 'SE' or 'TR' or 'CZ' or 'CA' or 'IN' or 'RU'
     *
     * @var string
     */
    public $country;

    /**
     * Initially selected currency one of:'AED' or 'ARS' or 'AUD' or 'BGN' or 'BND' or
     * 'BOB' or 'BRL' or 'CAD' or 'CHF' or 'CLP' or 'CNY' or 'COP' or 'CSD' or 'CZK' or
     * 'DEM' or 'DKK' or 'EEK' or 'EGP' or 'EUR' or 'FJD' or 'FRF' or 'GBP' or 'HKD' or
     * 'HRK' or 'HUF' or 'IDR' or 'ILS' or 'INR' or 'JOD' or 'JPY' or 'KES' or 'KRW' or
     * 'LKR' or 'LTL' or 'MAD' or 'MTL' or 'MXN' or 'MYR' or 'NGN' or 'NOK' or 'NZD' or
     * 'PEN' or 'PHP' or 'PKR' or 'PLN' or 'ROL' or 'RON' or 'RSD' or 'RUB' or 'SAR' or
     * 'SEK' or 'SGD' or 'SIT' or 'SKK' or 'THB' or 'TND' or 'TRL' or 'TRY' or 'TWD' or
     * 'UAH' or 'USD' or 'UYU' or 'VEB' or 'VEF' or 'VND' or 'ZAR'
     *
     * @var string
     */
    public $currencyCode;

    /**
     * Customer email
     * Regex: ^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$
     *
     * @var string
     */
    public $email;

    /**
     * Any name for the merchant. Can also be used to supply a seller id
     * Regex: ^[0-9a-zA-Z|;_\/\^!\(\)\-]{1,200}$
     *
     * @var string
     */
    public $name;

    /**
     * URL that is called once the user completes the first mandatory step of registration
     * Regex: ^(https?):\/\/[^\s/$.?#].[^\s]*$
     *
     * @var string
     */
    public $registrationCallback;

    /**
     * @return bool
     */
    public function isValid(&$errorMessage)
    {
        $errorMessage = null;
        // Check Country
        if (!in_array($this->country, self::$validCountries, true)) {
            $errorMessage = 'Ländercode fehlerhaft.';
            return false;
        }
        // Check CurrencyCode
        if (!in_array($this->currencyCode, self::$validCurrencyCodes, true)) {
            $errorMessage = 'Währungscode fehlerhaft.';
            return false;
        }
        // Check E-Mail
        if (!preg_match('/^[a-zA-Z0-9.!#$%&\'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/', $this->email)) {
            $errorMessage = 'E-Mail Adresse hat ein falschen format.';
            return false;
        }
        // Check Name
        if (!preg_match('/^[0-9a-zA-Z|;_\/\^!\(\)\-]{1,200}$/', $this->name)) {
            $errorMessage = 'Name fehlerhaft: nur Zahlen, Buchstaben oder <pre> | ; _ / - </pre> verwenden.';
            return false;
        }
        // Check Registration Callback
        if ($this->registrationCallback !== null && !preg_match('/^(https?):\/\/[^\s/$.?#].[^\s]*$/', $this->registrationCallback)) {
            $errorMessage = "Registration Callback fehlerhaft. Bitte kontaktiere den Support: <a href='https://support.webstollen.de'>https://support.webstollen.de</a>";
            return false;
        }
        // Chack AdwordsID
        if ($this->adwordsId !== null && !preg_match('/^[0-9]+$/', $this->adwordsId)) {
            $errorMessage = 'AdWords ID darf nur Zahlen enthalten.';
            return false;
        }

        return true;
    }

    /**
     * @return \WS\EMarketing\Response\Signup
     */
    public function getResponseObject()
    {
        return new \WS\EMarketing\Response\Signup();
    }
}