<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 13.09.18
 * Time: 10:20
 */

namespace WS\EMarketing\Resource;


use DateTime;

class Product extends AbstractResource
{

    const AGEGROUP_NEWBORN = 'newborn';
    const AGEGROUP_INFANT = 'infant';
    const AGEGROUP_TODDLER = 'toddler';
    const AGEGROUP_KIDS = 'kids';
    const AGEGROUP_ADULT = 'adult';

    const AVAILABILITY_INSTOCK = 'in stock';
    const AVAILABILITY_OUTOFSTOCK = 'out of stock';
    const AVAILABLILITY_PREORDER = 'preorder';

    const CONDITION_NEW = 'new';
    const CONDITION_REFURBISHED = 'refurbished';
    const CONDITION_USED = 'used';

    const EEC_APPP = 'A+++';
    const EEC_APP = 'A++';
    const EEC_AP = 'A+';
    const EEC_A = 'A';
    const EEC_B = 'B';
    const EEC_C = 'C';
    const EEC_D = 'D';
    const EEC_E = 'E';
    const EEC_F = 'F';
    const EEC_G = 'G';

    const SIZESYSTEM_US = 'US';
    const SIZESYSTEM_GB = 'GB';
    const SIZESYSTEM_DE = 'DE';
    const SIZESYSTEM_FR = 'FR';
    const SIZESYSTEM_JP = 'JP';
    const SIZESYSTEM_IT = 'IT';
    const SIZESYSTEM_ES = 'ES';
    const SIZESYSTEM_NL = 'NL';
    const SIZESYSTEM_BR = 'BR';
    const SIZESYSTEM_AU = 'AU';
    const SIZESYSTEM_CH = 'CH';
    const SIZESYSTEM_AT = 'AT';
    const SIZESYSTEM_BE = 'BE';
    const SIZESYSTEM_DK = 'DK';
    const SIZESYSTEM_MX = 'MX';
    const SIZESYSTEM_NO = 'NO';
    const SIZESYSTEM_PL = 'PL';
    const SIZESYSTEM_SE = 'SE';
    const SIZESYSTEM_TR = 'TR';
    const SIZESYSTEM_CZ = 'CZ';
    const SIZESYSTEM_CA = 'CA';
    const SIZESYSTEM_IN = 'IN';
    const SIZESYSTEM_RU = 'RU';

    const SIZETYPE_REGULAR = 'regular';
    const SIZETYPE_PETITE = 'petite';
    const SIZETYPE_PLUS = 'plus';
    const SIZETYPE_BIGANDTALL = 'big and tall';
    const SIZETYPE_MATERNITY = 'maternity';

    const GENDER_MALE = 'male';
    const GENDER_FEMALE = 'female';
    const GENDER_UNISEX = 'unisex';

    public static $TargetCountries = ['US', 'GB', 'DE', 'FR', 'JP', 'IT',
        'ES', 'NL', 'BR', 'AU', 'CH', 'AT', 'BE', 'DK', 'MX', 'NO', 'PL',
        'SE', 'TR', 'CZ', 'CA', 'IN', 'RU'];
    /**
     * @example ["https://cdn.example.org/gjdsf828dong2857/wooden-clubs-extra-large-2.png"]
     * @var string[]|null
     */
    public $additionalImageLinks;
    /**
     * Flag indicating if the product is targeted at an adult audience
     *
     * @var bool|null
     */
    public $adult;
    /**
     * Target age group one of:"newborn" or "infant" or "toddler" or "kids" or "adult" or null
     *
     * @var string|null
     */
    public $ageGroup;
    /**
     * Availability one of:"in stock" or "out of stock" or "preorder" or null
     *
     * @var string|null
     */
    public $availability;
    /**
     * The day a pre-ordered product becomes available for delivery,
     * in ISO 8601 format
     * Regex: ^([0-9]{4})(-)?(1[0-2]|0[1-9])(?(2)-)(3[0-1]|0[1-9]|[1-2][0-9])$
     *
     * @var DateTime|null
     */
    public $availabilityDate;
    /**
     * Brand
     *
     * @var string|null
     */
    public $brand;
    /**
     * @example ["cat123"]
     * @var string[]
     */
    public $categories;
    /**
     * @example "Brown"
     * @var string|null
     */
    public $color;
    /**
     * Condition one of:"new" or "refurbished" or "used" or null
     *
     * @var string|null
     */
    public $condition;
    /**
     * The two-letter ISO 639-1 language code for the product. Names,
     * descriptions and so on should be written in this language
     *
     * @example "en"
     * @var string
     */
    public $contentLanguage;
    /**
     * The international 3-letter code as defined by the ISO 4217 standard
     * @example "EUR"
     * @var string|null
     */
    public $currency;
    /**
     * Description of the product
     *
     * @example "This is the most exquisite wooden club ever devised"
     * @var string|null
     */
    public $description;
    /**
     * @TODO Whats that for?
     *
     * @example [{"destination_name":"Shopping","intention":"excluded"}]
     * @var Destination[]|null
     */
    public $destinations;
    /**
     * The energy efficiency class as defined in EU directive 2010/30/EU one
     * of:"A" or "A+" or "A++" or "A+++" or "B" or "C" or "D" or "E" or "F" or "G" or null
     *
     * @var string|null
     */
    public $energyEfficiencyClass;
    /**
     * Target gender one of:"male" or "female" or "unisex" or null
     *
     * @var string|null;
     */
    public $gender;
    /**
     * Category string (full path) based on Google's product taxonomy. IDs are also allowed
     *
     * @example "Arts & Entertainment > Hobbies & Creative Arts > Collectibles > Collectible Weapons"
     * @var string|null
     */
    public $googleCategory;
    /**
     * Global Trade Item Number
     *
     * @example "07350053850019"
     * @var string|null
     */
    public $gtin;
    /**
     * Must be unique across all of your products
     *
     * @example "clubs23-2"
     * @var string
     */
    public $id;
    /**
     * URL of the main product image. Must be publicly accessible pattern: ^(https?):\/\/[^\s/$.?#].[^\s]*$
     *
     * @example "https://cdn.example.org/gjdsf828dong2857/wooden-clubs-extra-large-1.png"
     * @var string|null
     */
    public $imageUrl;
    /**
     * Whether the item is a merchant-defined bundle.
     * A bundle is a custom grouping of different products
     * sold by a merchant for a single price
     *
     * @var bool|null
     */
    public $isBundle;
    /**
     * Margin in percent
     *
     * @example 3.4
     * @var float|null
     */
    public $margin;
    /**
     * Material
     *
     * @example "Wood"
     * @var string|null
     */
    public $material;
    /**
     * Manufacturer part number
     *
     * @var string|null
     */
    public $mpn;
    /**
     * The number of identical products in a merchant-defined multipack
     *
     * @example 3
     * @var int|null
     */
    public $multipack;
    /**
     * Primary name
     *
     * @example "Wooden club extra large"
     * @var string|null
     */
    public $name;
    /**
     * If this product is a variant, specify its parent product ID here
     *
     * @example "clubs23"
     * @var string|null
     */
    public $parentId;
    /**
     * Pattern
     *
     * @var string|null
     */
    public $pattern;
    /**
     * Current effective price without shipping, without discounts and including taxes
     *
     * @example 99.99
     * @var float|null
     */
    public $price;
    /**
     * The unique ID of a promotion
     *
     * @var string|null
     */
    public $promotionId;
    /**
     * Recommended retail price
     *
     * @var float|null
     */
    public $rrp;
    /**
     * Current effective sale price without shipping and including taxes
     *
     * @var float|null
     */
    public $salePrice;
    /**
     * Date range during which the item is on sale pattern: `^([+-]?\d{4}(?!\d{2}\b))((-?)((0[1-9]
     *
     * @example "2016-02-24T13:00-0800/2016-02-29T15:30-0800"
     * @var string|null
     */
    public $salePriceEffectiveDate;
    /**
     * Number of Sales through several platforms
     *
     * @var Sales|null
     */
    public $sales;
    /**
     * Shipping costs
     *
     * @example [{"price":4.99,"country":"DE","service":"Express"}]
     * @var Shipping[]
     */
    public $shipping = [];
    /**
     * The dimension of the product used to calculate the shipping cost of the item
     *
     * @var UnitValue|null
     */
    public $shippingHeight;
    /**
     * The dimension of the product used to calculate the shipping cost of the item
     *
     * @var UnitValue|null
     */
    public $shippingLength;
    /**
     * The weight of the product used to calculate the shipping cost of the item
     *
     * @var UnitValue|null
     */
    public $shippingWeight;
    /**
     * The dimension of the product used to calculate the shipping cost of the item
     *
     * @var UnitValue|null
     */
    public $shippingWidth;
    /**
     * Size
     *
     * @example "XXL"
     * @var string|null
     */
    public $size;
    /**
     * System in which the size is specified. Recommended for apparel items
     * one of:"AU" or "BR" or "CN" or "DE" or "EU" or "FR" or "IT" or "JP"
     * or "MEX" or "UK" or "US" or null
     *
     * @var string|null
     */
    public $sizeSystem;
    /**
     * The cut of the item. Recommended for apparel items one
     * of:"regular" or "petite" or "plus" or "big and tall" or "maternity" or null
     *
     * @var string|null
     */
    public $sizeType;
    /**
     * Stock quantity
     *
     * @var int
     */
    public $stockQuantity;
    /**
     * The country this item should be advertised in.
     * If this product is supposed to be advertised in multiple countries,
     * return the item multiple times with a different target_country and
     * other adjusted attributes one of:"US" or "GB" or "DE" or "FR" or "JP"
     * or "IT" or "ES" or "NL" or "BR" or "AU" or "CH" or "AT" or "BE" or "DK"
     * or "MX" or "NO" or "PL" or "SE" or "TR" or "CZ" or "CA" or "IN" or "RU"
     *
     * @var string
     */
    public $targetCountry;
    /**
     * - The unit of the denominator
     * - The denominator of the unit price
     *
     * @var UnitValue|null
     */
    public $unitPricingBaseMeasure;
    /**
     * - The unit of the measure
     * - The measure of an item
     *
     * @var UnitValue|null
     */
    public $unitPricingMeasure;
    /**
     * URL of the product page. Must be publicly accessible
     * pattern: ^(https?):\/\/[^\s/$.?#].[^\s]*$
     *
     * @var string
     */
    public $url;

    /**
     * @return bool
     */
    public function isValid(&$errorMessage)
    {

        $nullableStrings = [
            'brand' => $this->brand,
            'color' => $this->color,
            'description' => $this->description,
            'google_category' => $this->googleCategory,
            'gtin' => $this->gtin,
            'material' => $this->material,
            'mpn' => $this->mpn,
            'name' => $this->name,
            'parentId' => $this->parentId,
            'pattern' => $this->pattern,
            'promotionId' => $this->promotionId,
            'size' => $this->size,
        ];
        foreach ($nullableStrings as $key => $value) {
            if (!$this->_isValidString($key, $value, $errorMessage, true)) {
                $this->$key = null;
                //return false;
            }
        }

        $nullableNumbers = [
            'margin' => $this->margin,
            'multipack' => $this->multipack,
            'price' => $this->price,
            'rrp' => $this->rrp,
            'salePrice' => $this->salePrice,
        ];
        foreach ($nullableNumbers as $key => $value) {
            if (!$this->_isValidNumber($key, $value, $errorMessage, true)) {
                $this->$key = null;
                //return false;
            }
        }

        $strings = [
            'id' => $this->id,
        ];
        foreach ($strings as $key => $value) {
            if (!$this->_isValidString($key, $value, $errorMessage, false)) {
                return false;
            }
        }

        $nullableBools = [
            'adult' => $this->adult,
            'is_bundle' => $this->isBundle,
        ];
        foreach ($nullableBools as $key => $value) {
            if (!$this->_isValidBool($key, $value, $errorMessage, true)) {
                $this->$key = null;
                //return false;
            }
        }

        $arrays = [
            // '' => ['value' => '', 'nullable' => true, 'set' => false],
            'additionalImageLinks' => ['value' => $this->additionalImageLinks, 'nullable' => true, 'set' => false],
            'categories' => ['value' => $this->categories, 'nullable' => true, 'set' => false],
            'shipping' => ['value' => $this->shipping, 'nullable' => true, 'set' => false],

        ];
        foreach ($arrays as $key => $array) {
            if (!$this->_isValidArray($key, $array['value'], $errorMessage, $array['nullable'])) {
                if ($array['nullable'] === true) {
                    $this->$key = null;
                } else {
                    return false;
                }
            }
        }

        $sets = [
            'condition' => ['value' => $this->condition, 'nullable' => true, 'set' => [self::CONDITION_USED, self::CONDITION_REFURBISHED, self::CONDITION_NEW]],
            'energyEfficiencyClass' => ['value' => $this->energyEfficiencyClass, 'nullable' => true, 'set' => self::getECCArray()],
            'ageGroup' => ['value' => $this->ageGroup, 'nullable' => true, 'set' => [self::AGEGROUP_ADULT, self::AGEGROUP_INFANT, self::AGEGROUP_KIDS, self::AGEGROUP_NEWBORN, self::AGEGROUP_TODDLER]],
            'availability' => ['value' => $this->availability, 'nullable' => true, 'set' => [self::AVAILABLILITY_PREORDER, self::AVAILABILITY_OUTOFSTOCK, self::AVAILABILITY_INSTOCK]],
            'gender' => ['value' => $this->gender, 'nullable' => true, 'set' => [self::GENDER_FEMALE, self::GENDER_MALE, self::GENDER_UNISEX]],
            'targetCountry' => ['value' => $this->targetCountry, 'nullable' => false, 'set' => self::$TargetCountries],
            'sizeType' => ['value' => $this->sizeType, 'nullable' => true, 'set' => [self::SIZETYPE_BIGANDTALL, self::SIZETYPE_MATERNITY, self::SIZETYPE_PETITE, self::SIZETYPE_PLUS, self::SIZETYPE_REGULAR]],
            'sizeSystem' => ['value' => $this->sizeSystem, 'nullable' => true, 'set' => self::getSizeSystemArray()],
        ];
        foreach ($sets as $key => $set) {
            if (!$this->_isValidSet($key, $set['value'], $errorMessage, $set['set'], $set['nullable'])) {
                if ($set['nullable'] === true) {
                    $this->$key = null;
                } else {
                    return false;
                }
            }
        }

        $regexStrings = [
            // '' => ['value' => '', 'nullable' => true, 'pattern' => '/.*/'],
            'availabilityDate' => ['value' => $this->availabilityDate, 'nullable' => true, 'format' => 'YYYY-MM-DD', 'pattern' => '/^([0-9]{4})(-)?(1[0-2]|0[1-9])(?(2)-)(3[0-1]|0[1-9]|[1-2][0-9])$/'],
            'contentLanguage' => ['value' => $this->contentLanguage, 'nullable' => false, 'pattern' => '/^[a-z]{2}$/', 'format' => 'ISO 639-1'],
            'currency' => ['value' => $this->currency, 'nullable' => true, 'pattern' => '/^[A-Z]{3}$/'],
            'imageUrl' => ['value' => $this->imageUrl, 'nullable' => true, 'pattern' => '/^(https?):\/\/[^\s\/$.?#].[^\s]*$/'],
            'salePriceEffectiveDate' => ['value' => $this->salePriceEffectiveDate, 'nullable' => true, 'pattern' => '/^([\+-]?\d{4}(?!\d{2}\b))((-?)((0[1-9]|1[0-2])(\3([12]\d|0[1-9]|3[01]))?|W([0-4]\d|5[0-2])(-?[1-7])?|(00[1-9]|0[1-9]\d|[12]\d{2}|3([0-5]\d|6[1-6])))([T\s]((([01]\d|2[0-3])((:?)[0-5]\d)?|24\:?00)([\.,]\d+(?!:))?)?(\17[0-5]\d([\.,]\d+)?)?([zZ]|([\+-])([01]\d|2[0-3]):?([0-5]\d)?)?)?)?(\/)([\+-]?\d{4}(?!\d{2}\b))((-?)((0[1-9]|1[0-2])(\3([12]\d|0[1-9]|3[01]))?|W([0-4]\d|5[0-2])(-?[1-7])?|(00[1-9]|0[1-9]\d|[12]\d{2}|3([0-5]\d|6[1-6])))([T\s]((([01]\d|2[0-3])((:?)[0-5]\d)?|24\:?00)([\.,]\d+(?!:))?)?(\17[0-5]\d([\.,]\d+)?)?([zZ]|([\+-])([01]\d|2[0-3]):?([0-5]\d)?)?)?)?$/', 'format' => 'YYY-MM-DD'],
            'url' => ['value' => $this->url, 'nullable' => false, 'pattern' => '/^(https?):\/\/[^\s\/$.?#].[^\s]*$/'],
        ];
        foreach ($regexStrings as $key => $regexString) {
            if (!$this->_isValidRegexString($key, $regexString['value'], $errorMessage, $regexString['pattern'], $regexString['nullable'])) {
                if ($regexString['nullable'] === true) {
                    $this->$key = null;
                } else {
                    return false;
                }
            }
        }

        $objects = [
            'destination' => ['value' => $this->destinations, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\Destination'],
            'sales' => ['value' => $this->sales, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\Sales'],
            'shippingHeight' => ['value' => $this->shippingHeight, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
            'shippingLength' => ['value' => $this->shippingLength, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
            'shippingWeight' => ['value' => $this->shippingWeight, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
            'shippingWidth' => ['value' => $this->shippingWidth, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
            'unitPricingBaseMeasure' => ['value' => $this->unitPricingBaseMeasure, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
            'unitPricingMeasure' => ['value' => $this->unitPricingMeasure, 'nullable' => true, 'class' => 'WS\EMarketing\Resource\UnitValue'],
        ];
        foreach ($objects as $key => $object) {
            if (!$this->_isValidObject($key, $object['value'], $object['class'], $errorMessage, $object['nullable'])) {
                if ($object['nullable'] === true) {
                    $this->$key = null;
                } else {
                    return false;
                }
            }
        }


        return true;
    }

    /**
     * Returns all available EnergyEfficencyClasses
     *
     * @return array
     */
    public static function getECCArray()
    {
        return [
            self::EEC_APPP,
            self::EEC_APP,
            self::EEC_AP,
            self::EEC_A,
            self::EEC_B,
            self::EEC_C,
            self::EEC_D,
            self::EEC_E,
            self::EEC_F,
            self::EEC_G,
        ];
    }

    /**
     * Returns all available SizeSystems
     *
     * @return array
     */
    public static function getSizeSystemArray()
    {
        return [
            self::SIZESYSTEM_US,
            self::SIZESYSTEM_GB,
            self::SIZESYSTEM_DE,
            self::SIZESYSTEM_FR,
            self::SIZESYSTEM_JP,
            self::SIZESYSTEM_IT,
            self::SIZESYSTEM_ES,
            self::SIZESYSTEM_NL,
            self::SIZESYSTEM_BR,
            self::SIZESYSTEM_AU,
            self::SIZESYSTEM_CH,
            self::SIZESYSTEM_AT,
            self::SIZESYSTEM_BE,
            self::SIZESYSTEM_DK,
            self::SIZESYSTEM_MX,
            self::SIZESYSTEM_NO,
            self::SIZESYSTEM_PL,
            self::SIZESYSTEM_SE,
            self::SIZESYSTEM_TR,
            self::SIZESYSTEM_CZ,
            self::SIZESYSTEM_CA,
            self::SIZESYSTEM_IN,
            self::SIZESYSTEM_RU,
        ];
    }

}

